# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class CloudToDeviceMethod(Model):
    """Parameters to execute a direct method on the device.

    :param method_name: Method to run
    :type method_name: str
    :param payload: Payload
    :type payload: object
    :param response_timeout_in_seconds:
    :type response_timeout_in_seconds: int
    :param connect_timeout_in_seconds:
    :type connect_timeout_in_seconds: int
    """

    _attribute_map = {
        "method_name": {"key": "methodName", "type": "str"},
        "payload": {"key": "payload", "type": "object"},
        "response_timeout_in_seconds": {"key": "responseTimeoutInSeconds", "type": "int"},
        "connect_timeout_in_seconds": {"key": "connectTimeoutInSeconds", "type": "int"},
    }

    def __init__(self, **kwargs):
        super(CloudToDeviceMethod, self).__init__(**kwargs)
        self.method_name = kwargs.get("method_name", None)
        self.payload = kwargs.get("payload", None)
        self.response_timeout_in_seconds = kwargs.get("response_timeout_in_seconds", None)
        self.connect_timeout_in_seconds = kwargs.get("connect_timeout_in_seconds", None)
