# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class ServiceStatistics(Model):
    """ServiceStatistics.

    :param connected_device_count:
    :type connected_device_count: long
    """

    _attribute_map = {"connected_device_count": {"key": "connectedDeviceCount", "type": "long"}}

    def __init__(self, **kwargs):
        super(ServiceStatistics, self).__init__(**kwargs)
        self.connected_device_count = kwargs.get("connected_device_count", None)
