# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class RegistryStatistics(Model):
    """RegistryStatistics.

    :param total_device_count:
    :type total_device_count: long
    :param enabled_device_count:
    :type enabled_device_count: long
    :param disabled_device_count:
    :type disabled_device_count: long
    """

    _attribute_map = {
        "total_device_count": {"key": "totalDeviceCount", "type": "long"},
        "enabled_device_count": {"key": "enabledDeviceCount", "type": "long"},
        "disabled_device_count": {"key": "disabledDeviceCount", "type": "long"},
    }

    def __init__(self, **kwargs):
        super(RegistryStatistics, self).__init__(**kwargs)
        self.total_device_count = kwargs.get("total_device_count", None)
        self.enabled_device_count = kwargs.get("enabled_device_count", None)
        self.disabled_device_count = kwargs.get("disabled_device_count", None)
