# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.service_client import SDKClient
from msrest import Configuration, Serializer, Deserializer
from .version import VERSION
from msrest.exceptions import HttpOperationError
from .operations.service_operations import ServiceOperations
from . import models


class IotHubGatewayServiceAPIs20190630Configuration(Configuration):
    """Configuration for IotHubGatewayServiceAPIs20190630
    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param credentials: Subscription credentials which uniquely identify
     client subscription.
    :type credentials: None
    :param str base_url: Service URL
    """

    def __init__(self, credentials, base_url=None):

        if credentials is None:
            raise ValueError("Parameter 'credentials' must not be None.")
        if not base_url:
            base_url = "https://fully-qualified-iothubname.azure-devices.net"

        super(IotHubGatewayServiceAPIs20190630Configuration, self).__init__(base_url)

        self.add_user_agent("iothubgatewayserviceapis20190630/{}".format(VERSION))

        self.credentials = credentials


class IotHubGatewayServiceAPIs20190630(SDKClient):
    """IotHubGatewayServiceAPIs20190630

    :ivar config: Configuration for client.
    :vartype config: IotHubGatewayServiceAPIs20190630Configuration

    :ivar service: Service operations
    :vartype service: protocol.operations.ServiceOperations

    :param credentials: Subscription credentials which uniquely identify
     client subscription.
    :type credentials: None
    :param str base_url: Service URL
    """

    def __init__(self, credentials, base_url=None):

        self.config = IotHubGatewayServiceAPIs20190630Configuration(credentials, base_url)
        super(IotHubGatewayServiceAPIs20190630, self).__init__(self.config.credentials, self.config)

        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self.api_version = "2019-06-30"
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

        self.service = ServiceOperations(
            self._client, self.config, self._serialize, self._deserialize
        )
