# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.serialization import Model


class DeviceRegistryOperationWarning(Model):
    """Encapsulates device registry operation error details.

    :param device_id: The ID of the device that indicated the warning.
    :type device_id: str
    :param warning_code: Possible values include:
     'DeviceRegisteredWithoutTwin'
    :type warning_code: str or ~protocol.models.enum
    :param warning_status: Additional details associated with the warning.
    :type warning_status: str
    """

    _attribute_map = {
        "device_id": {"key": "deviceId", "type": "str"},
        "warning_code": {"key": "warningCode", "type": "str"},
        "warning_status": {"key": "warningStatus", "type": "str"},
    }

    def __init__(self, **kwargs):
        super(DeviceRegistryOperationWarning, self).__init__(**kwargs)
        self.device_id = kwargs.get("device_id", None)
        self.warning_code = kwargs.get("warning_code", None)
        self.warning_status = kwargs.get("warning_status", None)
